/******************************************
 *       Routine fr Dog-M Displays       *    
 *    mit ST7036 Controller im SPI-Mode   *
 ******************************************
 *                                        *
 * Nur 1-Zeilige Version vorerst...       *
 * Beschaltung laut Datenblatt EA-DOG-M   *
 *                                        *
 * (C)2007 Christopher Mller             *
 * shorty@shortysfastlane.de              *
 *                                        *
 * www.shortysfastlane.de                 *
 *                                        *
 * Stand: 17.Okt.2007                     *
 ******************************************/




// Vernderliche Variablen
//*******************************************
#define F_CPU    2000000UL  //Clockspeed in Hz
#define DDR_SPI  DDRB  // DD-Register des SPI-Ports
#define DD_MOSI  DDB3  // Port/Pin des MOSI
#define DD_SCK   DDB5  // Port/Pin des SCK
#define DD_RS    DDRC  // DDR von RS
#define PORT_RS  PORTC // Port an dem RS ist
#define PIN_RS   PC5   // Pin an dem RS ist
#define DD_CSB   DDRC  // DDR von CSB
#define PORT_CSB PORTC // Port an dem CSB ist
#define PIN_CSB  PC4   // Pin an dem CSB ist
#define VOLTAGE  5     // Voltage auf 5 oder 3 Volt
#define CONTRAST 0x7A  // Von 0x73 bis 0x7F Hexadezimal
//*******************************************
#include <util/delay.h>



void InitSPI (void)
{

	/* Set MOSI and SCK output, andere input */
	DDR_SPI = (1<<DD_MOSI)|(1<<DD_SCK);
	/* aktiviere SPI, Master, setze clock rate fck/16 */
	SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPR0);

	// Setze CSB als Ausgang und low
	DD_CSB |= (1<<PIN_CSB);
	PORT_CSB &= ~(1 << PIN_CSB);

	// Setze RS-Pin als Ausgang und low
	DD_RS |= (1<<PIN_RS);
	PORT_RS &= ~(1 << PIN_RS);

}



void Disp_Command(int Data)
{

	// Command Mode vorbeireiten; setze RS low
	PORT_RS &= ~(1 << PIN_RS);

	/* Starte bertragung */
	SPDR = Data;
	/* Warte auf ende der bertragung */
	while(!(SPSR & (1<<SPIF)));

	//bertragung abschlieen durch RS toggle
	PORT_RS = (1 << PIN_RS); // high
	PORT_RS &= ~(1 << PIN_RS); //low

	_delay_us(50); // warte auf Display
}



void Disp_Byte(int Data)
{
	// Data Mode vorbeireiten; setze RS high
	PORT_RS = (1 << PIN_RS);

	/* Starte bertragung */
	SPDR = Data;
	/* Warte auf ende der bertragung */
	while(!(SPSR & (1<<SPIF)));

	//bertragung abschlieen durch RS toggle
	PORT_RS &= ~(1 << PIN_RS); //low
	PORT_RS = (1 << PIN_RS); // high

	_delay_us(50); // warte auf Display
}



void Disp_Char(char cData)
{
	// Data Mode vorbeireiten; setze RS high
	PORT_RS = (1 << PIN_RS);

	/* Starte bertragung */
	SPDR = cData;
	/* Warte auf ende der bertragung */
	while(!(SPSR & (1<<SPIF)));

	//bertragung abschlieen durch RS toggle
	PORT_RS &= ~(1 << PIN_RS); //low
	PORT_RS = (1 << PIN_RS); // high

	_delay_us(50); // warte auf Display
}



void Disp_String(char *cData)
{
	// Data Mode vorbeireiten; setze RS high
	PORT_RS = (1 << PIN_RS);

	while (*cData)
	{
		/* Starte bertragung */
		SPDR = *cData;

		/* Warte auf ende der bertragung */
		while(!(SPSR & (1<<SPIF)));

		//bertragung abschlieen durch RS toggle
		PORT_RS &= ~(1 << PIN_RS); //low
		PORT_RS = (1 << PIN_RS); // high

		_delay_us(50); // warte auf Display
		++cData;
		}
}



void Disp_Clear(void)
{

	Disp_Command(0x01); // Disoplay lschen
	_delay_ms(25);      // Auf Display warten
	Disp_Command(0x06); // Entry Mode set - Display wartet auf Eingabe
	_delay_ms(15);      // Auf Display warten

}



void InitDisp(void)
{
	_delay_ms(80); // Warte auf Display

	int BIAS,PWR,FOLLOW;

	if (VOLTAGE == 5)
		{
		 BIAS = 0x1c;
		 PWR  = 0x51;
		 FOLLOW = 0x6a; 
		}
	else
		{
		 BIAS = 0x14;
		 PWR  = 0x55;
		 FOLLOW = 0x6d;
		}

	Disp_Command(0x31); // Function Set ; 8 Bit; 1Zeile, Istr.Tab 1
	Disp_Command(BIAS); // Bias Set 
	Disp_Command(PWR); // Power Controll
	Disp_Command(0x6a); // Follower Controll
	Disp_Command(CONTRAST); // Kontrast setzten
	Disp_Command(0x30); // Instruction Table 0 setzten
	Disp_Command(0x0c); // Display ein, kein Cursor, kein Blinken
	Disp_Command(0x01); // Disoplay lschen
	_delay_ms(25);      // Auf Display warten
	Disp_Command(0x06); // Entry Mode set - Display wartet auf Eingabe
	_delay_ms(15);      // Auf Display warten

}
